
/*
 *
 * This code is a reproduction of standard TI code


 * Copyright (c) 2020, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <lab3_helper.h>

void delay_cycles(uint32_t cycles)
{
    /* this is a scratch register for the compiler to use */
    uint32_t scratch;

    /* There will be a 2 cycle delay here to fetch & decode instructions
     * if branch and linking to this function */

    /* Subtract 2 net cycles for constant offset: +2 cycles for entry jump,
     * +2 cycles for exit, -1 cycle for a shorter loop cycle on the last loop,
     * -1 for this instruction */

    __asm volatile(
#ifdef __GNUC__
        ".syntax unified\n\t"
#endif
        "SUBS %0, %[numCycles], #2; \n"
        "%=: \n\t"
        "SUBS %0, %0, #4; \n\t"
        "NOP; \n\t"
        "BHS  %=b;" /* branches back to the label defined above if number > 0 */
        /* Return: 2 cycles */
        : "=&r"(scratch)
        : [ numCycles ] "r"(cycles));
}


void EnableLFCLKOutput(void) {
    // Enable external output of LFCLK signal.
    // Powers on GPIOA. This could be redundant!

    update_reg(&SYSCTL->SOCLOCK.GENCLKCFG,
        (uint32_t) SYSCTL_GENCLKCFG_EXCLKDIVEN_PASSTHRU | (uint32_t) SYSCTL_GENCLKCFG_EXCLKSRC_LFCLK,
        SYSCTL_GENCLKCFG_EXCLKDIVEN_MASK | SYSCTL_GENCLKCFG_EXCLKDIVVAL_MASK |
            SYSCTL_GENCLKCFG_EXCLKSRC_MASK);
    SYSCTL->SOCLOCK.GENCLKEN |= SYSCTL_GENCLKEN_EXCLKEN_ENABLE;

    GPIOA->GPRCM.RSTCTL = (GPIO_RSTCTL_KEY_UNLOCK_W | GPIO_RSTCTL_RESETSTKYCLR_CLR | GPIO_RSTCTL_RESETASSERT_ASSERT);
    GPIOA->GPRCM.PWREN = (GPIO_PWREN_KEY_UNLOCK_W | GPIO_PWREN_ENABLE_ENABLE);


    IOMUX->SECCFG.PINCM[(IOMUX_PINCM6)] = (IOMUX_PINCM_PC_CONNECTED | IOMUX_PINCM6_PF_SYSCTL_CLK_OUT);
    GPIOA->DOESET31_0 = 0x1u<<31; // PA31 is our output pin for the clock
}

void InitializeSleepIsStandby(void) {
    SYSCTL->SOCLOCK.PMODECFG = SYSCTL_PMODECFG_DSLEEP_STANDBY;
    SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;
    SYSCTL->SOCLOCK.MCLKCFG &= ~(SYSCTL_MCLKCFG_STOPCLKSTBY_ENABLE);
}

void InitializeProcessor(void) {
    SYSCTL->SOCLOCK.BORTHRESHOLD = SYSCTL_SYSSTATUS_BORCURTHRESHOLD_BORMIN; // Brownout generates a reset.

    update_reg(&SYSCTL->SOCLOCK.MCLKCFG, (uint32_t) SYSCTL_MCLKCFG_UDIV_NODIVIDE, SYSCTL_MCLKCFG_UDIV_MASK); // Set UPCLK divider
    update_reg(&SYSCTL->SOCLOCK.SYSOSCCFG, SYSCTL_SYSOSCCFG_FREQ_SYSOSCBASE, SYSCTL_SYSOSCCFG_FREQ_MASK); // Set SYSOSC to 32 MHz
//
//    // Set MCLK to LFCLK
//    SYSCTL->SOCLOCK.SYSOSCCFG |= SYSCTL_SYSOSCCFG_DISABLE_ENABLE; // Disable SYSOSC. We don't need it!
//    SYSCTL->SOCLOCK.MCLKCFG |= SYSCTL_MCLKCFG_USELFCLK_ENABLE;
//
//    // Verify LFCLK -> MCLK
//    while ((SYSCTL->SOCLOCK.CLKSTATUS & SYSCTL_CLKSTATUS_CURMCLKSEL_MASK) != SYSCTL_CLKSTATUS_CURMCLKSEL_LFCLK) {
//        ;
//    }

    // Disable MCLK Divider
    update_reg(&SYSCTL->SOCLOCK.MCLKCFG, (uint32_t) 0x0, SYSCTL_MCLKCFG_MDIV_MASK);

    EnableLFCLKOutput();
}


void InitializeTimerG0(void) {

    TIMG0->GPRCM.RSTCTL = (GPIO_RSTCTL_KEY_UNLOCK_W | GPIO_RSTCTL_RESETSTKYCLR_CLR | GPIO_RSTCTL_RESETASSERT_ASSERT);
    TIMG0->GPRCM.PWREN = (GPIO_PWREN_KEY_UNLOCK_W | GPIO_PWREN_ENABLE_ENABLE);
    delay_cycles(POWER_STARTUP_DELAY); // delay to enable GPIO to turn on and reset

    // Configure clocking for Timer Module
    TIMG0->CLKSEL = GPTIMER_CLKSEL_LFCLK_SEL_ENABLE;
    TIMG0->CLKDIV = GPTIMER_CLKDIV_RATIO_DIV_BY_1;

    /* This will configure what happens when we count down to zero - we'll set counter to LOAD value */

    TIMG0->COUNTERREGS.CC_01[0] = 0; // Count down to zero (value in CC0), then reload
    TIMG0->COUNTERREGS.CCCTL_01[0] = GPTIMER_CCCTL_01_ACOND_TIMCLK; // Set timer to advance on TIMCLK ticks

    // CM_DOWN - count down mode; REPEAT_1 - keep going when we reach zero; CVAE_LDVAL - when we get to zero, reload LOAD value;
    // EN_DISABLED - Start with timer disabled
    TIMG0->COUNTERREGS.CTRCTL =
        (( GPTIMER_CTRCTL_CVAE_LDVAL | GPTIMER_CTRCTL_CM_DOWN | GPTIMER_CTRCTL_REPEAT_REPEAT_1) | GPTIMER_CTRCTL_EN_DISABLED);


    // Enable timer interrupt when we reach 0
    TIMG0->CPU_INT.IMASK |= GPTIMER_CPU_INT_IMASK_Z_SET;

    TIMG0->PDBGCTL = GPTIMER_PDBGCTL_SOFT_IMMEDIATE;

    TIMG0->COMMONREGS.CCLKCTL = (GPTIMER_CCLKCTL_CLKEN_ENABLED);

    /* 
     * If you want to  manually test timing using debugging as in Lab 2, change TIMG0 to TIMA0 above
     * and uncomment the following code.

     * In main(), you need to change all instances of TIMG0 to TIMA0
     * 1. NVIC_EnableIRQ(TIMG0_INT_IRQn); ->  NVIC_EnableIRQ(TIMA0_INT_IRQn);
     * 2. TIMG0->COUNTERREGS.LOAD = DELAY; -> TIMA0->COUNTERREGS.LOAD = DELAY; 
     * 3. TIMG0->COUNTERREGS.CTRCTL |= (GPTIMER_CTRCTL_EN_ENABLED); ->
     *       TIMA0->COUNTERREGS.CTRCTL |= (GPTIMER_CTRCTL_EN_ENABLED);
     * 4. void TIMA0_IRQHandler(void) -> void TIMG0_IRQHandler(void)
     * 5. switch (TIMG0->CPU_INT.IIDX) -> switch (TIMA0->CPU_INT.IIDX) {
     * 
     */
    /*
    TIMA0->PDBGCTL = GPTIMER_PDBGCTL_SOFT_IMMEDIATE; 


    TIMA1->GPRCM.RSTCTL = (GPIO_RSTCTL_KEY_UNLOCK_W | GPIO_RSTCTL_RESETSTKYCLR_CLR | GPIO_RSTCTL_RESETASSERT_ASSERT);
    TIMA1->GPRCM.PWREN = (GPIO_PWREN_KEY_UNLOCK_W | GPIO_PWREN_ENABLE_ENABLE);
    delay_cycles(POWER_STARTUP_DELAY); // delay to enable GPIO to turn on and reset

    TIMA1->PDBGCTL = GPTIMER_PDBGCTL_SOFT_IMMEDIATE;
    TIMA1->CLKSEL = GPTIMER_CLKSEL_LFCLK_SEL_ENABLE;
    TIMA1->CLKDIV = GPTIMER_CLKDIV_RATIO_DIV_BY_1;
    TIMA1->COUNTERREGS.LOAD = 0xFFFF; // Reload to 32 bit max uint
    TIMA1->COUNTERREGS.CC_01[0] = 0; // Count down to zero (value in CC0), then reload
    TIMA1->COUNTERREGS.CCCTL_01[0] = GPTIMER_CCCTL_01_ACOND_TIMCLK; // Set timer to advance on TIMCLK ticks
    TIMA1->COUNTERREGS.CTRCTL =
        ((GPTIMER_CTRCTL_CVAE_LDVAL | GPTIMER_CTRCTL_CM_DOWN | GPTIMER_CTRCTL_REPEAT_REPEAT_1) | GPTIMER_CTRCTL_EN_ENABLED);

    TIMA1->COMMONREGS.CCLKCTL = (GPTIMER_CCLKCTL_CLKEN_ENABLED);
*/

}


